import 'package:flutter/material.dart';

import 'decoration/border.dart';
import 'decoration/border_radius.dart';
import 'decoration/shadow.dart';
import 'layout/alignment.dart';
import 'layout/fitted.dart';
import 'layout/flex.dart';
import 'opacity/opacity.dart';
import 'primitives/color.dart';
import 'primitives/hide.dart';
import 'primitives/size.dart';
import 'primitives/space.dart';
import 'rotate/rotate.dart';

/// Margin all
MarginAttribute m(double space) => MarginAttribute(
      left: space,
      right: space,
      top: space,
      bottom: space,
    );

/// Margin top
MarginAttribute mt(double space) => MarginAttribute(top: space);

/// Margin bottom
MarginAttribute mb(double space) => MarginAttribute(bottom: space);

/// Margin right
MarginAttribute mr(double space) => MarginAttribute(right: space);

/// Margin left

MarginAttribute ml(double space) => MarginAttribute(left: space);

/// Margin horizontal
MarginAttribute mx(double space) => MarginAttribute(
      left: space,
      right: space,
    );

/// Margin verttical
MarginAttribute my(double space) => MarginAttribute(top: space, bottom: space);

/// Padding all
PaddingAttribute p(double space) => PaddingAttribute(
      left: space,
      right: space,
      top: space,
      bottom: space,
    );

/// Padding top
PaddingAttribute pt(double space) => PaddingAttribute(top: space);

/// Padding bottom
PaddingAttribute pb(double space) => PaddingAttribute(bottom: space);

/// Padding right
PaddingAttribute pr(double space) => PaddingAttribute(right: space);

/// Padding left

PaddingAttribute pl(double space) => PaddingAttribute(left: space);

/// Padding horizontal
PaddingAttribute px(double space) => PaddingAttribute(
      left: space,
      right: space,
    );

/// Padding verttical
PaddingAttribute py(double space) =>
    PaddingAttribute(top: space, bottom: space);

/// Rotate widget
RotateAttribute rotate(int quarterTurns) => RotateAttribute(quarterTurns);

/// Rotate widget 90 degrees
final rotate90 = rotate(1);

/// Rotate widget 180 degrees
final rotate180 = rotate(2);

/// Rotate widget 270 degrees
final rotate270 = rotate(3);

/// Rotate widget 360 degrees
final rotate360 = rotate(4);

/// Opacity
OpacityAttribute opacity(double opacity) => OpacityAttribute(opacity);

/// Flex
FlexAttribute flex([int flex = 1]) => FlexAttribute(flex);

/// Flexible
final flexible = FlexFitAttribute(FlexFit.loose);

/// Expand
final expanded = FlexFitAttribute(FlexFit.tight);

/// Fitting
final fit = FittedUtility();

/// Decoration
/// Border radius
// final borderRadius = BorderRadiusCircularUtility();
final rounded = radiusFromParams;
BorderRadiusAttribute roundedTL(double radius) =>
    BorderRadiusAttribute.topLeft(radius);
BorderRadiusAttribute roundedTR(double radius) =>
    BorderRadiusAttribute.topRight(radius);
BorderRadiusAttribute roundedBL(double radius) =>
    BorderRadiusAttribute.bottomLeft(radius);
BorderRadiusAttribute roundedBR(double radius) =>
    BorderRadiusAttribute.bottomRight(radius);

/// Background color attribute
BackgroundColorAttribute backgroundColor(Color color) =>
    BackgroundColorAttribute(color);
final bg = backgroundColor;

/// Text color attribute
TextColorAttribute textColor(Color color) => TextColorAttribute(color);
final tc = textColor;

/// Shadow color attribute
ShadowColorAttribute shadowColor(Color color) => ShadowColorAttribute(color);
final sc = shadowColor;

/// Aligment
final align = AlignmentUtility();
final center = align.center;

final alignTL = align.topLeft;
final alignTC = align.topCenter;
final alignTR = align.topRight;

final alignBL = align.bottomLeft;
final alignBC = align.bottomCenter;
final alignBR = align.bottomRight;

final alignCL = align.centerLeft;
final alignC = align.center;
final alignCR = align.centerRight;

/// Constraints

/// Height
HeightAttribute h(double size) => HeightAttribute(size);

/// Width
WidthAttribute w(double size) => WidthAttribute(size);

/// Max height attribute
MaxHeightAttribute maxH(double size) => MaxHeightAttribute(size);

/// Min height attribute
MinHeightAttribute minH(double size) => MinHeightAttribute(size);

/// Max width attribute
MaxWidthAttribute maxW(double size) => MaxWidthAttribute(size);

/// Min width attribute
MinWidthAttribute minW(double size) => MinWidthAttribute(size);

GapAttribute gap(double size) => GapAttribute(size);

/// Hide attribute
// ignore: avoid_positional_boolean_parameters
HideAttribute hide(bool condition) => HideAttribute(hide: condition);

/// Border color for all borde sides
BorderAttribute borderColor(Color color) => BorderAttribute.all(
      BorderSideAttribute(color: color),
    );

/// Border width for all border sides
BorderAttribute borderWidth(double width) => BorderAttribute.all(
      BorderSideAttribute(width: width),
    );

/// Border style for all border sides
BorderAttribute borderStyle(BorderStyle style) => BorderAttribute.all(
      BorderSideAttribute(style: style),
    );
// final border = BorderUtility(BorderPosition.all);
// final b = border;

// final borderLeft = BorderUtility(BorderPosition.left);
// final bl = borderLeft;

// final borderTop = BorderUtility(BorderPosition.top);
// final bt = borderTop;

// final borderRight = BorderUtility(BorderPosition.right);
// final br = borderRight;

// final borderBottom = BorderUtility(BorderPosition.bottom);
// final bb = borderBottom;

// final borderColor = BorderUtility(BorderPosition.all).color;
// final bc = borderColor;

// final borderLeftColor = BorderUtility(BorderPosition.left).color;
// final blc = borderLeftColor;

// final borderTopColor = BorderUtility(BorderPosition.top).color;
// final btc = borderTopColor;

// final borderRightColor = BorderUtility(BorderPosition.right).color;
// final brc = borderRightColor;

// final borderBottomColor = BorderUtility(BorderPosition.bottom).color;
// final bbc = borderBottomColor;

// final borderWidth = BorderUtility(BorderPosition.all).width;
// final bw = borderWidth;

// final borderLeftWidth = BorderUtility(BorderPosition.left).width;
// final blw = borderLeftWidth;

// final borderTopWidth = BorderUtility(BorderPosition.top).width;
// final btw = borderTopWidth;

// final borderRightWidth = BorderUtility(BorderPosition.right).width;
// final brw = borderRightWidth;

// final borderBottomWidth = BorderUtility(BorderPosition.bottom).width;
// final bbw = borderBottomWidth;

// final borderStyle = BorderUtility(BorderPosition.all).style;
// final bs = borderStyle;

// final borderLeftStyle = BorderUtility(BorderPosition.left).style;
// final bls = borderLeftStyle;

// final borderTopStyle = BorderUtility(BorderPosition.top).style;
// final bts = borderTopStyle;

// final borderRightStyle = BorderUtility(BorderPosition.right).style;
// final brs = borderRightStyle;

// final borderBottomStyle = BorderUtility(BorderPosition.bottom).style;
// final bbs = borderBottomStyle;

final boxShadow = BoxShadowUtility();
final shadow = boxShadow;
final s = boxShadow;
