import 'package:flutter/material.dart';

import '../base_attribute.dart';

class BorderSideAttribute extends Attribute<BorderSide> {
  const BorderSideAttribute({
    this.color,
    this.width,
    this.style,
  });

  final Color? color;
  final double? width;
  final BorderStyle? style;

  static final _defaults = BorderSide(style: BorderStyle.solid);

  static BorderSideAttribute get none {
    return BorderSideAttribute(
      color: _defaults.color,
      width: _defaults.width,
      style: _defaults.style,
    );
  }

  BorderSide get value {
    return BorderSide(
      color: color ?? _defaults.color,
      width: width ?? _defaults.width,
      style: style ?? _defaults.style,
    );
  }

  BorderSideAttribute copyWith(BorderSideAttribute? other) {
    return BorderSideAttribute(
      color: other?.color ?? color,
      width: other?.width ?? width,
      style: other?.style ?? style,
    );
  }
}

/// Border attribute
class BorderAttribute extends Attribute<Border> {
  const BorderAttribute({
    this.left,
    this.top,
    this.right,
    this.bottom,
  });

  final BorderSideAttribute? left;
  final BorderSideAttribute? top;
  final BorderSideAttribute? right;
  final BorderSideAttribute? bottom;

  factory BorderAttribute.left(BorderSideAttribute side) {
    return BorderAttribute(
      left: side,
    );
  }

  factory BorderAttribute.top(BorderSideAttribute side) {
    return BorderAttribute(
      top: side,
    );
  }

  factory BorderAttribute.right(BorderSideAttribute side) {
    return BorderAttribute(
      right: side,
    );
  }

  factory BorderAttribute.bottom(BorderSideAttribute side) {
    return BorderAttribute(
      bottom: side,
    );
  }

  Border get value {
    return Border(
      left: left?.value ?? BorderSide.none,
      top: top?.value ?? BorderSide.none,
      right: right?.value ?? BorderSide.none,
      bottom: bottom?.value ?? BorderSide.none,
    );
  }

  factory BorderAttribute.all(BorderSideAttribute side) {
    return BorderAttribute(
      left: side,
      top: side,
      right: side,
      bottom: side,
    );
  }

  static BorderAttribute get none {
    return BorderAttribute.all(BorderSideAttribute.none);
  }

  BorderAttribute copyWith(BorderAttribute other) {
    return BorderAttribute(
      left: left?.copyWith(other.left),
      top: top?.copyWith(other.top),
      right: right?.copyWith(other.right),
      bottom: bottom?.copyWith(other.bottom),
    );
  }
}
