import 'package:flutter/material.dart';

import '../base_attribute.dart';

class _BoxShadowColorUtility {
  BoxShadowAttribute operator [](Color color) {
    return BoxShadowAttribute(color: color);
  }
}

class _BoxShadowOffsetUtility {
  BoxShadowAttribute operator [](Offset offset) {
    return BoxShadowAttribute(offset: offset);
  }
}

class _BoxShadowBlurRadiusUtility {
  BoxShadowAttribute operator [](double blurRadius) {
    return BoxShadowAttribute(blurRadius: blurRadius);
  }
}

class _BoxShadowSpreadRadiusUtility {
  BoxShadowAttribute operator [](double spreadRadius) {
    return BoxShadowAttribute(spreadRadius: spreadRadius);
  }
}

class BoxShadowUtility {
  _BoxShadowColorUtility get color => _BoxShadowColorUtility();
  _BoxShadowOffsetUtility get offset => _BoxShadowOffsetUtility();
  _BoxShadowBlurRadiusUtility get blurRadius => _BoxShadowBlurRadiusUtility();
  _BoxShadowSpreadRadiusUtility get spreadRadius =>
      _BoxShadowSpreadRadiusUtility();
}

class BoxShadowAttribute extends Attribute<BoxShadow> {
  const BoxShadowAttribute({
    this.color,
    this.offset,
    this.blurRadius,
    this.spreadRadius,
  });

  final Color? color;
  final Offset? offset;
  final double? blurRadius;
  final double? spreadRadius;

  static final _defaults = const BoxShadow();

  static BoxShadowAttribute get none {
    return BoxShadowAttribute(
      color: _defaults.color,
      offset: _defaults.offset,
      blurRadius: _defaults.blurRadius,
      spreadRadius: _defaults.spreadRadius,
    );
  }

  BoxShadow get value {
    return BoxShadow(
      color: color ?? _defaults.color,
      offset: offset ?? _defaults.offset,
      blurRadius: blurRadius ?? _defaults.blurRadius,
      spreadRadius: spreadRadius ?? _defaults.spreadRadius,
    );
  }

  BoxShadowAttribute copyWith(BoxShadowAttribute other) {
    return BoxShadowAttribute(
      color: other.color ?? color,
      offset: other.offset ?? offset,
      blurRadius: other.blurRadius ?? blurRadius,
      spreadRadius: other.spreadRadius ?? spreadRadius,
    );
  }
}
