import 'package:flutter/widgets.dart';

import '../components/gap_widget.dart';
import '../mixer/mix.dart';
import 'remix_widget.dart';

class FlexBox extends RemixWidget {
  FlexBox(
    Mix mix, {
    Key? key,
    required this.direction,
    required this.children,
  }) : super(mix, key: key);

  final List<Widget> children;
  final Axis direction;

  factory FlexBox.row(
    Mix mix, {
    Key? key,
    required List<Widget> children,
  }) {
    return FlexBox(
      mix,
      key: key,
      children: children,
      direction: Axis.horizontal,
    );
  }

  factory FlexBox.column(
    Mix mix, {
    Key? key,
    required List<Widget> children,
  }) {
    return FlexBox(
      mix,
      key: key,
      children: children,
      direction: Axis.vertical,
    );
  }

  // Creates gap to space in between
  List<Widget> renderChildrenWithGap(List<Widget> children) {
    // If no gap is set return widgets
    if (attr.gap == null) return children;

    // List of widgets with gap
    final widgets = <Widget>[];
    for (var idx = 0; idx < children.length; idx++) {
      final widget = children[idx];
      widgets.add(widget);
      // Add gap if not last item if its not last element

      if (widget != children.last) {
        widgets.add(GapWidget(attr.gap!.value));
      }
    }

    return widgets;
  }

  @override
  Widget build(BuildContext context) {
    return mix.box(
      child: Flex(
        direction: direction,
        mainAxisAlignment:
            attr.mainAxisAlignment?.value ?? MainAxisAlignment.start,
        crossAxisAlignment:
            attr.crossAxisAlignment?.value ?? CrossAxisAlignment.center,
        mainAxisSize: attr.mainAxisSize?.value ?? MainAxisSize.max,
        verticalDirection: VerticalDirection.down,
        children: renderChildrenWithGap(children),
      ),
    );
  }
}

class RowMix extends FlexBox {
  RowMix(
    Mix mix, {
    Key? key,
    List<Widget> children = const <Widget>[],
  }) : super(
          mix,
          key: key,
          children: children,
          direction: Axis.horizontal,
        );
}

class ColumnMix extends FlexBox {
  ColumnMix(
    Mix mix, {
    Key? key,
    List<Widget> children = const <Widget>[],
  }) : super(
          mix,
          key: key,
          children: children,
          direction: Axis.vertical,
        );
}
